import setuptools
import os
import sys
from setuptools.command.install import install


with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="nanogo-basecaller",
    version="0.1.5",
    author="Gurasis Osahan",
    author_email="gurasis.osahan@phac-aspc.gc.ca",
    description="A bioinformatics pipeline for basecalling ONT NGS Data.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/phac-nml/nanogo-basecaller",
    package_dir={"": "src"},
    package_data={"nanogo_basecaller": ["tools/basecaller/custom.html.j2"]},
    packages=setuptools.find_packages(where="src"),
    python_requires="<=3.11",
    install_requires=[
        # Core Utilities and Build Tools
        "build>=1.2.2.post1",
        "certifi>=2025.1.31",
        "charset-normalizer>=3.4.1",
        "exceptiongroup>=1.2.2",
        "platformdirs>=4.3.6",
        "packaging>=24.2",
        "wheel>=0.45.1",
        "poetry-core>=2.1.1",
        "pyproject-hooks>=1.2.0",
        # Scientific and Data Analysis Libraries
        "numpy<2.0.0,>=1.24.3",
        "scipy>=1.15.2",
        "pandas>=2.2.3",
        "polars>=0.20.31",
        "pyarrow>=14.0.2",
        "h5py>=3.10.0",
        "pillow>=11.1.0",
        "matplotlib>=3.10.1",
        "contourpy>=1.3.1",
        "kiwisolver>=1.4.8",
        "plotly>=4.1.0",
        "tqdm>=4.67.1",
        # Bioinformatics and Scientific Tools
        "bio>=1.7.1",
        "biopython>=1.85",
        "mygene>=3.2.2",
        "biothings-client>=0.4.1",
        "pycoQC>=2.5.2",
        "gprofiler-official>=1.0.0",
        "lib-pod5>=0.3.10",
        "pod5==0.3.10",
        "vbz-h5py-plugin>=1.0.1",
        # Programming and Utilities
        "cython>=3.0.12",
        "psutil>=7.0.0",
        "six>=1.17.0",
        "et-xmlfile>=2.0.0",
        "iso8601>=2.1.0",
        "python-dateutil>=2.9.0.post0",
        "pytz>=2025.1",
        "tzdata>=2025.1",
        # Networking and APIs
        "requests>=2.32.3",
        "httpcore>=1.0.7",
        "httpx>=0.28.1",
        "urllib3>=2.3.0",
        "idna>=3.10",
        # Authentication and Security
        "oauthlib>=3.2.2",
        "google-auth>=2.38.0",
        "grpcio>=1.71.0",
        # Web and Visualization
        "markdown>=3.7",
        "jinja2>=3.1.6",
        # Data Management
        "pooch>=1.8.2",
        "openpyxl>=3.1.5",
        # AI/ML Support
        "protobuf<3.20,>=3.9.2",
        # Monitoring and Debugging
        "gputil>=1.4.0",
        "sniffio>=1.3.1",
        "alive-progress>=3.2.0",
        # Terminal and Logging
        "about-time>=4.2.1",
        "h11>=0.14.0",
        "more-itertools>=10.6.0",
        "grapheme>=0.6.0",
        "cycler>=0.12.1",
        "typing-extensions>=4.12.2",
        # Development Tools
        "ipython>=8.34.0",
        "anyio>=4.9.0",
        "narwhals>=1.31.0",
    ],
    entry_points={
        "console_scripts": [
            "nanogo=nanogo_basecaller.cli:main",
            "install-dorado=nanogo_basecaller.dorado_installer:main",
        ]
    },
    keywords="nanopore sequencing ONT bioinformatics dorado basecalling NGS pipeline primer-trimming",
    license="GPL-3.0-or-later",
    project_urls={
        "Source": "https://github.com/phac-nml/nanogo-basecaller",
        "Tracker": "https://github.com/phac-nml/nanogo-basecaller/-/issues",
        "Documentation": "https://github.com/phac-nml/nanogo-basecaller/wiki",
        "Publication": "https://github.com/phac-nml/nanogo-basecaller#citation",
        "Contact": "https://github.com/phac-nml/nanogo-basecaller#contact",
    },
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "Intended Audience :: Science/Research",
        "Intended Audience :: Healthcare Industry",
        "Topic :: Scientific/Engineering :: Bio-Informatics",
        "Topic :: Scientific/Engineering :: Medical Science Apps.",
        "Topic :: Scientific/Engineering :: Artificial Intelligence",
        "Topic :: Scientific/Engineering :: Image Processing",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: System :: Distributed Computing",
        "Operating System :: OS Independent",
        "Operating System :: POSIX :: Linux",
        "Natural Language :: English",
        "Environment :: GPU :: NVIDIA CUDA",
        "Environment :: Console",
        "Framework :: Pytest",
        "License :: OSI Approved :: GNU General Public License (GPL)",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
    ],
)
